@@ -1,7 +1,7 @@
 package Bencher::Backend;
 
-our $DATE = '2016-03-24'; # DATE
-our $VERSION = '0.43'; # VERSION
+our $DATE = '2016-03-26'; # DATE
+our $VERSION = '0.44'; # VERSION
 
 use 5.010001;
 use strict;
@@ -872,7 +872,29 @@ sub _gen_items {
     [200, "OK", $items];
 }
 
-sub _complete_module {
+sub _complete_scenario_module {
+    my %args = @_;
+    my $word    = $args{word} // '';
+    my $cmdline = $args{cmdline};
+    my $r       = $args{r};
+
+    return undef unless $cmdline;
+
+    # force reading config file
+    $r->{read_config} = 1;
+    my $res = $cmdline->parse_argv($r);
+    my $args = $res->[2];
+
+    require Complete::Module;
+    {
+        local @INC = @INC;
+        unshift @INC, $_ for @{ $args->{include_path} // [] };
+        Complete::Module::complete_module(
+            word=>$args{word}, ns_prefix=>'Bencher::Scenario');
+    }
+}
+
+sub _complete_participant_module {
     my %args = @_;
     my $word    = $args{word} // '';
     my $cmdline = $args{cmdline};
@@ -1188,10 +1210,11 @@ sub format_result {
 
     $formatters //= [
         'AddVsSlowestField',
-        'RoundNumbers',
         ['Sort', {by=>$opts->{sort}}],
         'ScaleTime',
         'ScaleRate',
+        'ScaleSize',
+        'RoundNumbers',
         ($envres->[3]{'func.module_startup'} ? ('ModuleStartup') : ()),
         'DeleteConstantFields',
         'DeleteNotesFieldIfEmpty',
@@ -1262,12 +1285,7 @@ the scenario specification.
 _
             schema => ['str*', match=>qr!\A\w+((?:::|/)\w+)*\z!],
             cmdline_aliases => {m=>{}},
-            completion => sub {
-                require Complete::Module;
-                my %args = @_;
-                Complete::Module::complete_module(
-                    word=>$args{word}, ns_prefix=>'Bencher::Scenario');
-            },
+            completion => sub { _complete_scenario_module(@_) },
         },
         participants => {
             'summary' => 'Add participants',
@@ -1324,6 +1342,7 @@ _
                            list-items
                            show-items-codes
                            show-items-results
+                           show-items-results-sizes
                            bench
                        /]
                     # list-functions
@@ -1386,6 +1405,11 @@ _
                     summary => 'Shortcut for -a show-items-results',
                     code => sub { $_[0]{action} = 'show-items-results' },
                 },
+                show_items_results_sizes => {
+                    is_flag => 1,
+                    summary => 'Shortcut for -a show-items-results-sizes',
+                    code => sub { $_[0]{action} = 'show-items-results-sizes' },
+                },
             },
             tags => ['category:action'],
         },
@@ -1414,7 +1438,7 @@ _
             summary => 'Only include modules specified in this list',
             'summary.alt.plurality.singular' => 'Add module to include list',
             schema => ['array*', of=>['str*']],
-            element_completion => sub { _complete_module(@_, apply_filters=>0) },
+            element_completion => sub { _complete_participant_module(@_, apply_filters=>0) },
             tags => ['category:filtering'],
         },
         include_module_pattern => {
@@ -1427,7 +1451,7 @@ _
             summary => 'Exclude modules specified in this list',
             'summary.alt.plurality.singular' => 'Add module to exclude list',
             schema => ['array*', of=>['str*']],
-            element_completion => sub { _complete_module(@_, apply_filters=>0) },
+            element_completion => sub { _complete_participant_module(@_, apply_filters=>0) },
             tags => ['category:filtering'],
         },
         exclude_module_pattern => {
@@ -1658,7 +1682,7 @@ of the module in `@INC`. Then will generate items for each version.
 Currently only one module can be multi version.
 
 _
-            completion => sub { _complete_module(@_, apply_filters=>0) },
+            completion => sub { _complete_participant_module(@_, apply_filters=>0) },
         },
         include_path => {
             summary => 'Additional module search paths',
@@ -1708,6 +1732,16 @@ _
             default => ['-time'],
         },
 
+        include_result_size => {
+            summary => "Aside from time, also return each item result's memory usage",
+            schema => 'bool',
+            description => <<'_',
+
+Memory size is measured using `Devel::Size`.
+
+_
+        },
+
         return_meta => {
             summary => 'Whether to return extra metadata',
             description => <<'_',
@@ -1988,7 +2022,7 @@ sub bencher {
         goto L_END;
     }
 
-    if ($action =~ /\A(show-items-results|bench)\z/) {
+    if ($action =~ /\A(show-items-results-sizes|show-items-results|bench)\z/) {
         require Module::Load;
         require Time::HiRes;
 
@@ -2049,6 +2083,9 @@ sub bencher {
         my $on_failure = $args{on_failure} // $parsed->{on_failure} // 'die';
         my $on_result_failure = $args{on_result_failure} //
             $parsed->{on_result_failure} // $on_failure;
+        my $include_result_size = $args{include_result_size} //
+            $parsed->{include_result_size} // 0;
+        $include_result_size = 1 if $action eq 'show-items-results-sizes';
         {
             last if $args{multiperl} || $args{multimodver};
             my $fitems = [];
@@ -2104,6 +2141,13 @@ sub bencher {
             $items = $fitems;
         }
 
+        if ($include_result_size) {
+            require Devel::Size;
+            for my $it (@$items) {
+                $it->{_result_size} = Devel::Size::total_size($it->{_result});
+            }
+        }
+
         if ($action eq 'show-items-results') {
             die "show-items-results currently not supported on multiperl or multimodver\n" if $args{multiperl} || $args{multimodver};
             if ($return_meta) {
@@ -2131,6 +2175,23 @@ sub bencher {
             goto RETURN_RESULT;
         }
 
+        if ($action eq 'show-items-results-sizes') {
+            die "show-items-results currently not supported on multiperl or multimodver\n" if $args{multiperl} || $args{multimodver};
+            if ($return_meta) {
+                $envres->[2] = [map {$_->{_result_size}} @$items];
+            } else {
+                $envres->[2] = join(
+                    "",
+                    map {(
+                        "#$_->{seq} ($_->{_name}):\n",
+                        $_->{_result_size},
+                        "\n\n",
+                    )} @$items
+                );
+            }
+            goto RETURN_RESULT;
+        }
+
         if ($return_meta) {
             $envres->[3]{'func.bencher_version'} = $Bencher::VERSION;
             $envres->[3]{'func.bencher_args'} = {
@@ -2161,6 +2222,7 @@ sub bencher {
         my @rows;
         if ($args{multiperl} || $args{multimodver}) {
             require Data::Clone;
+            require Devel::Size;
             require File::Temp;
             my %perl_exes;
             my %perl_opts;
@@ -2201,7 +2263,7 @@ sub bencher {
                         "-MBencher::Backend",
                         "-MData::Dmp",
                         @{ $perl_opts{$modver} // [] },
-                        "-e'print dmp(Bencher::Backend::bencher(action=>q[bench], precision=>$precision, scenario_file=>q[$scd_path], return_meta=>0))' > '$res_path'",
+                        "-e'print dmp(Bencher::Backend::bencher(action=>q[bench], precision=>$precision, scenario_file=>q[$scd_path], include_result_size=>q[$include_result_size], return_meta=>0))' > '$res_path'",
                     );
                     $log->debugf("Running %s ...", $cmd);
                     system $cmd;
@@ -2241,10 +2303,13 @@ sub bencher {
             for my $seq (sort {$a<=>$b} keys %$tres) {
                 my $it = _find_record_by_seq($items, $seq);
                 my $row = {
-                    time    => $tres->{$seq}{result}{num},
                     rate    => 1 / $tres->{$seq}{result}{num},
-                    samples => $tres->{$seq}{result}{_dbr_nsamples},
+                    time    => $tres->{$seq}{result}{num},
+
+                    (result_size => $it->{_result_size}) x !!$include_result_size,
+
                     errors  => $tres->{$seq}{result}{errors}[0],
+                    samples => $tres->{$seq}{result}{_dbr_nsamples},
                     notes   => $it->{_code_error},
                 };
 
@@ -2257,7 +2322,9 @@ sub bencher {
             }
         }
 
-        push @columns, qw/seq rate time errors samples notes/;
+        push @columns, qw/seq rate time/;
+        push @columns, qw/result_size/ if $include_result_size;
+        push @columns, qw/errors samples notes/;
 
         $envres->[2] = \@rows;
         $envres->[3]{'table.fields'} = \@columns;
@@ -2348,7 +2415,7 @@ Bencher::Backend - Backend for Bencher
 
 =head1 VERSION
 
-This document describes version 0.43 of Bencher::Backend (from Perl distribution Bencher-Backend), released on 2016-03-24.
+This document describes version 0.44 of Bencher::Backend (from Perl distribution Bencher-Backend), released on 2016-03-26.
 
 =head1 FUNCTIONS
 
@@ -2489,6 +2556,12 @@ Used when searching for scenario module, or when in multimodver mode.
 
 Only include some perls.
 
+=item * B<include_result_size> => I<bool>
+
+Aside from time, also return each item result's memory usage.
+
+Memory size is measured using C<Devel::Size>.
+
 =item * B<module_startup> => I<bool>
 
 Benchmark module startup overhead instead of normal benchmark.
