@@ -1,7 +1,7 @@
 package Bencher::Formatter::RoundNumbers;
 
-our $DATE = '2016-03-24'; # DATE
-our $VERSION = '0.43'; # VERSION
+our $DATE = '2016-03-26'; # DATE
+our $VERSION = '0.44'; # VERSION
 
 use 5.010001;
 use strict;
@@ -10,6 +10,8 @@ use warnings;
 
 use parent qw(Bencher::Formatter);
 
+use Scalar::Util qw(looks_like_number);
+
 use Role::Tiny::With;
 with 'Bencher::Role::ResultMunger';
 
@@ -41,6 +43,12 @@ sub munge_result {
             $rit->{vs_slowest} = sprintf(
                 $fmt, $rit->{vs_slowest});
         }
+
+        for my $col (keys %$rit) {
+            if ($col =~ /^(result_size)$/) {
+                $rit->{$col} = sprintf($fmt, $rit->{$col});
+            }
+        }
     }
 }
 
@@ -59,7 +67,7 @@ Bencher::Formatter::RoundNumbers - Round numbers (rate, time) to certain signifi
 
 =head1 VERSION
 
-This document describes version 0.43 of Bencher::Formatter::RoundNumbers (from Perl distribution Bencher-Backend), released on 2016-03-24.
+This document describes version 0.44 of Bencher::Formatter::RoundNumbers (from Perl distribution Bencher-Backend), released on 2016-03-26.
 
 =for Pod::Coverage .*
 
